const {KafkaClient, Producer} = require('kafka-node');
const R = require('ramda');
const debug = require('debug')('post-json-to-kafka:debug');
const trace = require('debug')('post-json-to-kafka:trace');

const stringifyMessageTopic_ = (to, objectMessage) => R.pipe(
  R.assoc('to', to),
  JSON.stringify
)(objectMessage);

const formatAllMessages_ = (to, messages) => R.map(
  message => stringifyMessageTopic_(to, message),
  messages
);

const getPayload_ = (to, topic, messages) => R.pipe(
  () => R.objOf('messages', formatAllMessages_(to, messages)),
  R.assoc('topic', topic),
  R.of,
  R.tap(debug)
)();

const getOptionPartitionType_ = type => R.ifElse(
  () => R.has(type, Producer.PARTITIONER_TYPES),
  R.always(R.objOf('partitionerType', R.prop(type, Producer.PARTITIONER_TYPES))),
  R.always({})
)();

const sendToKafka =
  (topic, to, messages, partitionerType = 'default') =>
    new Promise((resolve, reject) => {
      trace('Topic : ', topic);
      trace('to : ', to);
      trace('messages : ', messages);
      const client = new KafkaClient({kafkaHost: process.env.KAFKA_URL});
      const producer = new Producer(client, getOptionPartitionType_(partitionerType));

      producer.on('ready', () => {
        debug('Producer ready');
        producer.send(getPayload_(to, topic, messages), () => {
          debug('Payload sent');
          client.close(resolve);
        });
      });
      producer.on('error', error =>
        reject(error));
    });

module.exports = {sendToKafka};
